

-- The Ada 95 Booch Components (Version 1.0 beta 1)
-- Copyright (C)1994-1997 Grady Booch and David Weller.  All Rights Reserved.
-- 
--      This program is free software; you can redistribute it
--      and/or modify it under the terms of the Ada Community
--      License which comes with this Library.
--
--      This program is distributed in the hope that it will be
--      useful, but WITHOUT ANY WARRANTY; without even the implied
--      warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR
--      PURPOSE. See the Ada Community License for more details.
--      You should have received a copy of the Ada Community
--      License with this library, in the file named "Ada Community
--      License" or "ACL". If not, contact the author of this library 
--      for a copy.
--
--  This file contains the declaration of various node types.

with Ada.Finalization;
generic
    type Item is private;
	-- real soon we must add a param here for mem mgr.
	-- with package Mem_Mgr is new Bc.Support.Redirectable_Pools;
package BC.Support.Nodes is

   type Node;
   type Node_Ref is access Node;
   type Node is tagged record
      Elem : aliased Item;
      Prev : Node_Ref;
      Next : Node_Ref;
   end record;

   type Single_Node;
   type Single_Node_Ref is access Single_Node;
   type Single_Node is record
      Elem  : aliased Item;
      Next  : Single_Node_Ref;
      Count : Natural :=1;
   end record;

   type Double_Node;
   type Double_Node_Ref is access Double_Node;
   type Double_Node is record
      Elem  : aliased Item;
      Prev  : Double_Node_Ref;
      Next  : Double_Node_Ref;
      Count : Natural :=1;
   end record;

-- Class denoting a simple node consisting of an item, a pointer to
-- the parent, pointers to the left and right items, and a reference count
   type Binary_Node;
   type Binary_Node_Ref is access Binary_Node;

   type Binary_Node is record
      Elem : aliased Item;
      Parent, Left, Right : Binary_Node_Ref;
      Count : Natural := 1;
   end record;


-- Class denoting a simple node consisting of an item, a pointer to
-- the parent, pointers to the child and sibling items, and a reference count
   type Multiway_Node;
   type Multiway_Node_Ref is access Multiway_Node;

   type Multiway_Node is record
      Elem : aliased Item;
      Parent, Child, Sibling : Multiway_Node_Ref;
      Count : Natural := 1;
   end record;

-- Class denoting a simple node consisting of an item, pointers to the
-- left and right items, and a balance
   type AVL_Node;
   type AVL_Node_Ref is access AVL_Node;
   type Balance_Values is (Left, Middle, Right);

   type AVL_Node is record
      Elem : aliased Item;
      Left, Right : AVL_Node_Ref;
      Balance : Balance_Values := Middle;
   end record;

end BC.Support.Nodes;
